var files = [];
var md5s = [];
var dhashes = [];

function handleFileSelect(target) {
    cleanup();
    files = target.files;
    document.getElementById('status').textContent = 'Loading images ...';
    loadFile(0);
}

function handleExport() {
    document.getElementById('status').textContent = 'Processing images ...';
    loadMd5(0);
}

function loadFile(i) {
    dhashes[i] = [];
    // Only process image files.
    if (files[i].type.match('image.*')) {
        imageReader = new FileReader();
        imageReader.readAsDataURL(files[i]);
        imageReader.onload = (function (file, n) {
            return function (e) {
                im = new Image();
                im.src = e.target.result;
                imageData = getImageData(im, 8, 8);

                dhashes[n] = dHash(imageData.data);

                renderThumbnail(file.name, dhashes[n], im.src, n, files.length);

                if (n + 1 < files.length) {
                    loadFile(n + 1);
                } else {
                    doCompare();
                }
            };
        })(files[i], i);
    } else if (i + 1 < files.length) {
        loadFile(i + 1);
    } else {
        doCompare();
    }
}

function loadMd5(i) {
    md5s[i] = null;
    // Only process image files.
    if (files[i].type.match('image.*')) {
        md5reader = new FileReader();
        md5reader.readAsArrayBuffer(files[i]);
        md5reader.onload = (function (n) {
            return function (e) {
                wordArray = CryptoJS.lib.WordArray.create(e.target.result);
                md5s[n] = CryptoJS.MD5(wordArray);
                document.getElementById('status').textContent = 'Processed ' + (n + 1) + '/' + files.length + ' md5s ...';
                if (n + 1 < files.length) {
                    loadMd5(i + 1);
                } else {
                    doExport();
                }
            };
        })(i);
    } else if (i + 1 < files.length) {
        loadMd5(i + 1);
    } else {
        doExport();
    }
}

function getImageData(im, width, height) {
    // Render in canvas
    canv = document.createElement('canvas');
    canv.width = width;
    canv.height = height;
    ctx = canv.getContext('2d');
    ctx.drawImage(im, 0, 0, width, height);

    // Get all canvas pixel data
    return ctx.getImageData(0, 0, width, height);
}

function renderThumbnail(name, dhash, src, n, num) {
    div = document.createElement('div');
    div.className = 'fig';
    div.id = 'fig-' + n;
    div.innerHTML = '<img class="thumb" src="' + src + '" title="dHash: ' + toHexHash(dhash) + '" onclick="handleClick(this)"/><br>&nbsp;&nbsp;' + escape(name) + '&nbsp;&nbsp;';
    document.getElementById('thumbs').insertBefore(div, null);
    document.getElementById('status').textContent = 'Loaded ' + (n + 1) + '/' + num + ' images ...';
}

function cleanup() {
    files = [];
    md5s = [];
    dhashes = [];

    fig = document.getElementsByClassName('fig');
    while (fig.length) {
        fig[0].parentNode.removeChild(fig[0]);
    }
    figsel = document.getElementsByClassName('figsel');
    while (figsel.length) {
        figsel[0].parentNode.removeChild(figsel[0]);
    }
    res = document.getElementById('res');
    if (res) {
        res.parentNode.removeChild(res);
    }

    document.getElementsByName('export')[0].disabled = true;
}

function handleClick(img) {
    if (window.event.metaKey || window.event.ctrlKey) {
        EXIF.getData(img, function () {
            newWindow = window.open(null, 'Exif', 'resizable = yes');
            newWindow.document.write('<font face="Arial">' + EXIF.pretty(this) + '</font>');
            newWindow.document.close();
        });
    } else if (window.event.shiftKey) {
        EXIF.getData(img, function () {
            lat = convertCoords(EXIF.getTag(this, 'GPSLatitude'), EXIF.getTag(this, 'GPSLatitudeRef'));
            lon = convertCoords(EXIF.getTag(this, 'GPSLongitude'), EXIF.getTag(this, 'GPSLongitudeRef'));
            url = 'http://maps.googleapis.com/maps/api/staticmap?size=640x640&maptype=roadmap&sensor=false&center=' + lat + ',' + lon + '&zoom=5&markers=color:red%7C' + lat + ',' + lon;
            window.open(url, 'Map', 'resizable=yes');
        });
    } else {
        newWindow = window.open(null, 'Image', 'resizable=yes');
        image = newWindow.document.createElement('img');
        image.src = img.src;
        newWindow.document.body.appendChild(image);
    }
}

function handleOver(n, k) {
    document.getElementById('fig-' + n).className = 'figover';
    document.getElementById('fig-' + k).className = 'figover';
}

function handleOut(n, k) {
    document.getElementById('fig-' + n).className = 'figsel';
    document.getElementById('fig-' + k).className = 'figsel';
}

function convertCoords(coord, ref) {
    conv = coord[0] + coord[1] / 60 + coord[2] / (60 * 60);
    if (ref === 'S' || ref === 'W') {
        conv = -conv;
    }
    return conv;
}

function doCompare() {
    varianceTolerance = 16;
    numFiles = files.length;
    result = '<b>Selected images are visually similar</b><table>';

    for (n = 0; n < numFiles; n++) {
        for (k = n + 1; k < numFiles; k++) {
            dist = distance(dhashes[n], dhashes[k]);
            if (dist <= varianceTolerance) {
                result += '<tr onmouseover="handleOver(' + n + ', ' + k + ')"  onmouseout="handleOut(' + n + ', ' + k + ')"><td>' + files[n].name + '</td><td>&nbsp;&nbsp;&hArr;&nbsp;&nbsp;</td><td>' + files[k].name + '</td><td>&nbsp;&nbsp;:&nbsp;&nbsp;' + dist + '</td></tr>';
                document.getElementById('fig-' + n).className = 'figsel';
                document.getElementById('fig-' + k).className = 'figsel';
            }
        }
    }
    result += '</table>';
    span = document.createElement('span');
    span.id = 'res';
    span.innerHTML = result;
    document.getElementById('result').insertBefore(span, null);
    document.getElementById('status').textContent = 'Click to open Image, Shift-Click to open Map, Ctrl-Click to open Exif, Mouse-Over to view dHash';
    document.getElementsByName('export')[0].disabled = false;
}

function doExport() {
    data = '';
    for (n = 0; n < files.length; n++) {
        data += files[n].name + ',' + md5s[n].toString() + ',' + toHexHash(dhashes[n]) + '\n';
    }
    anchor = document.createElement('a');
    anchor.setAttribute('href', 'data:text/csv,' + encodeURIComponent(data));
    anchor.setAttribute('download', 'export.csv');
    anchor.click();
    document.getElementById('status').textContent = 'Click to open Image, Shift-Click to open Map, Ctrl-Click to open Exif, Mouse-Over to view dHash';
}

